﻿/**
 * Designed to run some simple animation benchmark #'s to see how the particular 
 * hardware and flashplayer configuration play together. If you have any comments
 * suggestions, or code contributions, please e-mail me and I will gladly add them
 * if appropriate.
 * 
 * Author:
 *  Nate Chatellier, e-mail: natejc@natejc.com
 * 
 * Usage:
 *  It should be fairly straight forward. Just drag the sliders to add a new ball
 *  that is animated in the specified method. You can click on the top half or bottom
 *  half of the DiagnosticStats window to increase or decrease the max fps. Use the
 *  + or - keys to increase/decrease the maximum # of balls allowed. Press Q or X
 *  to quit.
 * 
 * Changing the Resolution:
 *  1) Change the resolution in the actual .fla file itself.
 *  2) Change the size of _mcBG to match this resolution exactly.
 *  3) Change the width of _mcPanel to the new resolution.
 *  4) Change the middle keyframe of BallTimeline so that the x location of ball is
 *     the new horizontal resolution - the width of the ball.
 *  5) Realign any controls as desired.
 * 
 * Version log:
 *  08.19.2008		1.0		Nate Chatellier	+ First version
 * 
 * @see	http://blog.natejc.com
 * 
 * Released under the MIT License:
 * Copyright (c) 2008 Nate Chatellier
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package 
{
	import flash.display.Sprite;
	import flash.display.MovieClip;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.text.TextField;
	import flash.ui.Keyboard;
	import flash.events.KeyboardEvent;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.BlurFilter;
	import flash.filters.BevelFilter;
	import flash.filters.DropShadowFilter;
	
	import fl.controls.CheckBox;
	import fl.controls.Slider;
	import fl.events.SliderEvent;
	
	import com.natejc.diagnostics.DiagnosticStats;
	
	import caurina.transitions.Tweener;
	
	
// **********************************************************************************
// **********************************************************************************


	/**
	 * Drives the BenchmarkTest fla's.
	 * 
	 * @author Nate Chatellier
	 */
	public class BenchmarkTest_fp9 extends Sprite
	{
		public static var	BALL_STEP_SIZE_MIN	:Number = 2;
		public static var	BALL_STEP_SIZE_MAX	:Number = 40;
		
		public static var	BALL_TWEEN_TIME_MIN	:Number = 0.3;
		public static var	BALL_TWEEN_TIME_MAX	:Number = 4;
		
		public var		_mcPanel				:Sprite;
		public var		_mcBG					:Sprite;
		
		public var		_slBallsEnterFrame		:Slider;
		public var		_slBallsTweener			:Slider;
		public var		_slBallsTimeline		:Slider;
		public var		_cbToggleAlpha			:CheckBox;
		public var		_cbToggleBevel			:CheckBox;
		public var		_cbToggleShadow			:CheckBox;
		public var		_cbToggleBlur			:CheckBox;
		
		public var		_txtNumBallsEnterFrame	:TextField;
		public var		_txtNumBallsTweener		:TextField;
		public var		_txtNumBallsTimeline	:TextField;
		
		protected var	_mcStats				:DiagnosticStats;
		
		protected var	_aBallsEnterFrame		:Array;
		protected var	_aBallsTweener			:Array;
		protected var	_aBallsTimeline			:Array;
		protected var	_aBallStep				:Array;
		
		protected var	_nNumBallsEnterFrame	:int;
		protected var	_nNumBallsTweener		:int;
		protected var	_nNumBallsTimeline		:int;
		
		protected var	_bfBlurFilter			:BlurFilter;
		protected var	_bfBevelFilter			:BevelFilter;
		protected var	_bfShadowFilter			:DropShadowFilter;
		
		
	// **********************************************************************************
		
		
		/**
		 * Constructs the BenchmarkTest_fp9 object.
		 */
		public function BenchmarkTest_fp9()
		{			
			_mcStats = new DiagnosticStats();
			_mcStats.x = _mcBG.width - _mcStats.width;
			this.addChild(_mcStats);

			this.stage.align		= StageAlign.TOP_LEFT;
			this.stage.scaleMode	= StageScaleMode.NO_SCALE;

			_mcPanel.mouseChildren	= false;
			_mcPanel.mouseEnabled	= false;
			
			_aBallsEnterFrame	= new Array();
			_aBallsTweener		= new Array();
			_aBallsTimeline		= new Array();
			_aBallStep			= new Array();
			
			_nNumBallsEnterFrame	= 0;
			_nNumBallsTweener		= 0;
			_nNumBallsTimeline		= 0;
			
			_slBallsEnterFrame.liveDragging	= true; // TODO: not sure why this isn't working...
			_slBallsTweener.liveDragging	= true; // TODO: not sure why this isn't working...
			_slBallsTimeline.liveDragging	= true; // TODO: not sure why this isn't working...
			
			_slBallsEnterFrame.addEventListener(SliderEvent.CHANGE,	sliderEnterFrameChangeEvent,	false, 0, true)
			_slBallsTweener.addEventListener(SliderEvent.CHANGE,	sliderTweenerChangeEvent,		false, 0, true)
			_slBallsTimeline.addEventListener(SliderEvent.CHANGE,	sliderTimelineChangeEvent,		false, 0, true)
			
			_cbToggleAlpha.addEventListener(MouseEvent.CLICK,	checkBoxClickedHandler, false, 0, true);
			_cbToggleBevel.addEventListener(MouseEvent.CLICK,	checkBoxClickedHandler, false, 0, true);
			_cbToggleShadow.addEventListener(MouseEvent.CLICK,	checkBoxClickedHandler, false, 0, true);
			_cbToggleBlur.addEventListener(MouseEvent.CLICK,	checkBoxClickedHandler, false, 0, true);

			_bfBlurFilter	= new BlurFilter(30, 30, 1);
			_bfBevelFilter	= new BevelFilter(5, 135, 0xFFFFFF, .8, 0x000000, .8, 10, 10);
			_bfShadowFilter	= new DropShadowFilter(10, 135, 0, .8, 8, 8);
			
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDownHandler, false, 0, true);
		} // END CONSTRUCTOR
		
		
	// **********************************************************************************
	
	
		/**
		 * Adds a new ball that is updated by an EnterFrame event to the stage with any applicable (checked) filters.
		 */
		protected function addEnterFrameBall():void
		{
			if (_nNumBallsEnterFrame == 0 && !this.hasEventListener(Event.ENTER_FRAME))
				this.addEventListener(Event.ENTER_FRAME, enterFrameEvent, false, 0, true);

			_aBallsEnterFrame[_nNumBallsEnterFrame] = new BallEnterFrame();
			Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).mouseChildren	= false;
			Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).mouseEnabled	= false;
			_aBallStep[_nNumBallsEnterFrame] = Math.floor(Math.random() * (BALL_STEP_SIZE_MAX - BALL_STEP_SIZE_MIN) + 1);
			
			var nBallHeight:Number = Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).height;
			
			if (_nNumBallsEnterFrame > 0)
			{
				Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).y =	(Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame - 1]).y 
																	+ (nBallHeight * .1))
																	% (_mcBG.height - nBallHeight);
				if (Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).y < _mcPanel.height)
					Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).y = _mcPanel.height;
			}
			else 
				Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).y = _mcPanel.height;
			
			this.addChild(_aBallsEnterFrame[_nNumBallsEnterFrame]);
			
			updateEffects( Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]) );
			
			_nNumBallsEnterFrame++;
			
		} // END FUNCTION addEnterFrameBall
	
	
	// **********************************************************************************
	
	
		/**
		 * Removes the last EnterFrame ball added.
		 */
		protected function removeEnterFrameBall():void
		{
			_nNumBallsEnterFrame--;
			
			if (_aBallsEnterFrame[_nNumBallsEnterFrame])
			{
				if (Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).parent)
					Sprite(_aBallsEnterFrame[_nNumBallsEnterFrame]).parent.removeChild(_aBallsEnterFrame[_nNumBallsEnterFrame]);
				_aBallsEnterFrame[_nNumBallsEnterFrame] = null;
			} // END IF
			
			if (_nNumBallsEnterFrame <= 0)
			{
				this.removeEventListener(Event.ENTER_FRAME, enterFrameEvent, false);
				_nNumBallsEnterFrame = 0;
			} // END IF
			
		} // END FUNCTION removeEnterFrameBall
		
		
	// **********************************************************************************
	
	
		/**
		 * Updates the location of the balls to make them appear that they are moving.
		 *
		 * @param	e	The dispatched Event.ENTER_FRAME event.
		 */
		protected function enterFrameEvent(e:Event):void
		{
			for (var i:uint = 0; i < _nNumBallsEnterFrame; i++)
			{
				Sprite(_aBallsEnterFrame[i]).x += _aBallStep[i];
				if (Sprite(_aBallsEnterFrame[i]).x <= 0 || Sprite(_aBallsEnterFrame[i]).x + Sprite(_aBallsEnterFrame[i]).width >= _mcBG.width)
					_aBallStep[i] *= -1;
			} // END FOR
		} // END FUNCTION enterFrameEvent
	
	
	// **********************************************************************************
	
	
		/**
		 * Adds a new ball that is updated by Tweener to the stage with any applicable (checked) filters.
		 */
		protected function addTweenerBall():void
		{
			_aBallsTweener[_nNumBallsTweener] = new BallTweener();
			Sprite(_aBallsTweener[_nNumBallsTweener]).mouseChildren	= false;
			Sprite(_aBallsTweener[_nNumBallsTweener]).mouseEnabled	= false;
			Sprite(_aBallsTweener[_nNumBallsTweener]).y = _mcPanel.height;
			
			var nBallWidth:Number = Sprite(_aBallsTweener[_nNumBallsTweener]).width;
			
			if (_nNumBallsTweener > 0)
				Sprite(_aBallsTweener[_nNumBallsTweener]).x = (Sprite(_aBallsTweener[_nNumBallsTweener - 1]).x + (nBallWidth * .1)) % (_mcBG.width - nBallWidth);
			else 
				Sprite(_aBallsTweener[_nNumBallsTweener]).x = 0;
			
			this.addChild(_aBallsTweener[_nNumBallsTweener]);
			
			updateEffects( Sprite(_aBallsTweener[_nNumBallsTweener]) );
			
			updateTweenerBallMotion(_aBallsTweener[_nNumBallsTweener], "down");
			
			_nNumBallsTweener++;
			
		} // END FUNCTION addTweenerBall
	
	
	// **********************************************************************************
	
	
		/**
		 * Removes the last Tweener ball added.
		 */
		protected function removeTweenerBall():void
		{
			_nNumBallsTweener--;
			
			if (_nNumBallsTweener < 0)
				_nNumBallsTweener = 0;
			
			if (_aBallsTweener[_nNumBallsTweener])
			{
				Tweener.removeTweens(_aBallsTweener[_nNumBallsTweener]);
				if (Sprite(_aBallsTweener[_nNumBallsTweener]).parent)
					Sprite(_aBallsTweener[_nNumBallsTweener]).parent.removeChild(_aBallsTweener[_nNumBallsTweener]);
				_aBallsTweener[_nNumBallsTweener] = null;
			} // END IF
			
		} // END FUNCTION removeTweenerBall
	
	
	// **********************************************************************************
	
	
		/**
		 * A recursive Tweener call of sorts. This function oscillates the animation direction of the Tweener ball between up and down.
		 *
		 * @param	mcTweenerBall	The ball that is to be animated.
		 * @param	sDirection		The direction that the ball should be animating toward: either "UP" or "DOWN".
		 * @param	nTime			The amount of time in seconds that it will take the ball to reach one side of the screen.
		 */
		protected function updateTweenerBallMotion(mcTweenerBall:Sprite, sDirection:String, nTime:Number = -1):void
		{
			if (nTime <= 0)
				nTime = Math.random() * ((BALL_TWEEN_TIME_MAX - BALL_TWEEN_TIME_MIN) * 100 + 1) * .01 + BALL_TWEEN_TIME_MIN;
			
			var nYDestination:Number = (sDirection == "UP") ? _mcPanel.height : _mcBG.height - mcTweenerBall.height;
			var sNextDirection:String = (sDirection == "UP") ? "DOWN" : "UP";
			
			var oTweenAniBall:Object = 
			{
				y					:	nYDestination,
				time				:	nTime,
				transition			:	"linear",
				onComplete			:	updateTweenerBallMotion,
				onCompleteParams	:	[mcTweenerBall, sNextDirection, nTime]
			}; // END OBJECT
			Tweener.addTween(mcTweenerBall, oTweenAniBall);
			
		} // END FUNCTION updateTweenerBallMotion
	
	
	// **********************************************************************************
	
	
		/**
		 * Adds a new ball that is updated by an EnterFrame event to the stage with any applicable (checked) filters.
		 */
		protected function addTimelineBall():void
		{
			_aBallsTimeline[_nNumBallsTimeline] = new BallTimeline();
			Sprite(_aBallsTimeline[_nNumBallsTimeline]).mouseChildren	= false;
			Sprite(_aBallsTimeline[_nNumBallsTimeline]).mouseEnabled	= false;
			
			var nBallHeight:Number = Sprite(_aBallsTimeline[_nNumBallsTimeline]).height;
			
			if (_nNumBallsTimeline > 0)
			{
				Sprite(_aBallsTimeline[_nNumBallsTimeline]).y =	(Sprite(_aBallsTimeline[_nNumBallsTimeline - 1]).y 
																	+ (nBallHeight * .1))
																	% (_mcBG.height - nBallHeight);
				if (Sprite(_aBallsTimeline[_nNumBallsTimeline]).y < _mcPanel.height)
					Sprite(_aBallsTimeline[_nNumBallsTimeline]).y = _mcPanel.height;
			}
			else 
				Sprite(_aBallsTimeline[_nNumBallsTimeline]).y = _mcPanel.height;
			
			this.addChild(_aBallsTimeline[_nNumBallsTimeline]);
			
			updateEffects( Sprite(_aBallsTimeline[_nNumBallsTimeline]) );
			
			MovieClip(_aBallsTimeline[_nNumBallsTimeline]).gotoAndPlay( Math.floor(Math.random() * (MovieClip(_aBallsTimeline[_nNumBallsTimeline]).totalFrames - 1) + 1) );

			_nNumBallsTimeline++;
			
		} // END FUNCTION addTimelineBall
	
	
	// **********************************************************************************
	
	
		/**
		 * Removes the last EnterFrame ball added.
		 */
		protected function removeTimelineBall():void
		{
			_nNumBallsTimeline--;
			
			if (_aBallsTimeline[_nNumBallsTimeline])
			{
				MovieClip(_aBallsTimeline[_nNumBallsTimeline]).stop();
				if (Sprite(_aBallsTimeline[_nNumBallsTimeline]).parent)
					Sprite(_aBallsTimeline[_nNumBallsTimeline]).parent.removeChild(_aBallsTimeline[_nNumBallsTimeline]);
				_aBallsTimeline[_nNumBallsTimeline] = null;
			} // END IF
			
		} // END FUNCTION removeTimelineBall
	
	
	// **********************************************************************************
	
	
		/**
		 * Applies all checked effects to <code>mcTarget</code> or to all balls if <code>mcTarget == null</code>.
		 *
		 * @param	mcTarget	Specifies which ball should have their effects updated, or leave this parameter null to apply the effects to all balls.
		 */
		protected function updateEffects(mcTarget:Sprite = null):void
		{
			var aFilters:Array = new Array();
			if (_cbToggleBlur.selected)
				aFilters.push(_bfBlurFilter);
			if (_cbToggleBevel.selected)
				aFilters.push(_bfBevelFilter);
			if (_cbToggleShadow.selected)
				aFilters.push(_bfShadowFilter);
			
			if (mcTarget)
			{
				if (_cbToggleAlpha.selected)
					mcTarget.alpha = .5;
				else
					mcTarget.alpha = 1;
				
				mcTarget.filters = aFilters;
			}
			else
			{
				for (var i:uint = 0; i < _nNumBallsEnterFrame; i++)
				{
					Sprite(_aBallsEnterFrame[i]).alpha = (_cbToggleAlpha.selected) ? .5 : 1;
					Sprite(_aBallsEnterFrame[i]).filters = aFilters;
				} // END FOR
				
				for (i = 0; i < _nNumBallsTweener; i++)
				{
					Sprite(_aBallsTweener[i]).alpha = (_cbToggleAlpha.selected) ? .5 : 1;
					Sprite(_aBallsTweener[i]).filters = aFilters;
				} // END FOR
				
				for (i = 0; i < _nNumBallsTimeline; i++)
				{
					Sprite(_aBallsTimeline[i]).alpha = (_cbToggleAlpha.selected) ? .5 : 1;
					Sprite(_aBallsTimeline[i]).filters = aFilters;
				} // END FOR
			} // END IF
		} // END FUNCTION updateEffects
	
	
	// **********************************************************************************
	
	
		/**
		 * Called whenever the _slBallsEnterFrame slider is changed. This function updates the associated text 
		 * label _txtNumBallsEnterFrame with the new slider's value.
		 *
		 * @param	e	The dispatched SliderEvent.CHANGE event.
		 */
		protected function sliderEnterFrameChangeEvent(e:SliderEvent):void
		{
			_txtNumBallsEnterFrame.text	= String(e.value);
			
			while (e.value > _nNumBallsEnterFrame)
				this.addEnterFrameBall();
			
			while (e.value < _nNumBallsEnterFrame)
				this.removeEnterFrameBall();
			
		} // END FUNCTION sliderEnterFrameChangeEvent
		
		
	// **********************************************************************************
	
	
		/**
		 * Called whenever the _slBallsTweener slider is changed. This function updates the associated text 
		 * label _txtNumBallsTweener with the new slider's value.
		 *
		 * @param	e	The dispatched SliderEvent.CHANGE event.
		 */
		protected function sliderTweenerChangeEvent(e:SliderEvent):void
		{
			_txtNumBallsTweener.text	= String(e.value);
			
			while (e.value > _nNumBallsTweener)
				this.addTweenerBall();
			
			while (e.value < _nNumBallsTweener)
				this.removeTweenerBall();
			
		} // END FUNCTION sliderTweenerChangeEvent
		
		
	// **********************************************************************************
	
	
		/**
		 * Called whenever the _slBallsTimeline slider is changed. This function updates the associated text 
		 * label _txtNumBallsTimeline with the new slider's value.
		 *
		 * @param	e	The dispatched SliderEvent.CHANGE event.
		 */
		protected function sliderTimelineChangeEvent(e:SliderEvent):void
		{
			_txtNumBallsTimeline.text	= String(e.value);

			while (e.value > _nNumBallsTimeline)
				this.addTimelineBall();
			
			while (e.value < _nNumBallsTimeline)
				this.removeTimelineBall();
			
		} // END FUNCTION sliderTimelineChangeEvent
	
	
	// **********************************************************************************
	
	
		/**
		 * Called whenever a checkbox is toggled. It updates the applied effects on all balls appropriately.
		 *
		 * @param	e	This dispatched MouseEvent.CLICK event.
		 */
		protected function checkBoxClickedHandler(e:MouseEvent):void
		{
			updateEffects();
		} // END FUNCTION checkBoxClickedHandler

		
	// **********************************************************************************
		

		/**
		 * Called whenever a KeyboardEvent.KEY_DOWN event occurs.
		 *
		 * @param	e	This dispatched KeyboardEvent.KEY_DOWN event.
		 */
        protected function keyDownHandler(event:KeyboardEvent):void
		{
			switch(event.keyCode)
			{
				case Keyboard.NUMPAD_ADD:
				case 187:
					_slBallsEnterFrame.maximum	+= 10;
					_slBallsTweener.maximum		= _slBallsEnterFrame.maximum;
					_slBallsTimeline.maximum	= _slBallsEnterFrame.maximum;
				break;
				
				case Keyboard.NUMPAD_SUBTRACT:
				case 189:
					_slBallsEnterFrame.maximum	-= 10;
					_slBallsTweener.maximum		= _slBallsEnterFrame.maximum;
					_slBallsTimeline.maximum	= _slBallsEnterFrame.maximum;
				break;
					
			} // END SWITCH
			
        } // END FUNCTION keyDownHandler
		
		
// **********************************************************************************
// **********************************************************************************


	} // END CLASS BenchmarkTest_fp9
} // END PACKAGE


