﻿/**
 * Hi-ReS! DiagnosticStats v1.3
 * http://code.google.com/p/mrdoob/wiki/stats
 * 
 * Released under MIT license:
 * http://www.opensource.org/licenses/mit-license.php
 *  
 * How to use:
 * 
 * 	addChild( new DiagnosticStats() );
 * 
 * version log:
 *
 *  08.08.12		1.3		Nate Chatellier	+ Reformatted to match coding standards and renamed & repackaged.
 *	08.07.12		1.3		Mr.doob			+ Some speed and code optimisations
 *	08.02.15		1.2		Mr.doob			+ Class renamed to DiagnosticStats (previously FPS)
 *	08.01.05		1.2		Mr.doob			+ Click changes the fps of flash (half up increases,
 *									  		half down decreases)
 *	08.01.04		1.1		Mr.doob & Theo	+ Log shape for MEM
 * 											+ More room for MS
 * 											+ Shameless ripoff of Alternativa's FPS look ;)
 * 	07.12.13		1.0		Mr.doob			+ First version
 **/

package com.natejc.diagnostics
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Rectangle;
	import flash.system.System;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.utils.getTimer;
	
	
// **********************************************************************************
// **********************************************************************************

	
	public class DiagnosticStats extends Sprite
	{		
		private var _bdGraph	:BitmapData;
		
		private var _txtFps		:TextField;
		private var	_txtMs		:TextField;
		private var	_txtMem		:TextField;
		private var	_tfStyle	:TextFormat;
		
		private var _nFps		:int;
		private var	_nTimer		:int;
		private var	_nMs		:int;
		private var	_nMsPrev	:int	= 0;
		
		private var _nMem		:Number	= 0;

		
	// **********************************************************************************

	
		public function DiagnosticStats():void
		{
			_bdGraph = new BitmapData( 60, 50, false, 0x333 );
			var gBitmap:Bitmap = new Bitmap( _bdGraph );
			gBitmap.y = 35;
			addChild(gBitmap);
		
			_tfStyle = new TextFormat( "_sans", 9 );
			
			graphics.beginFill(0x333);
			graphics.drawRect(0, 0, 60, 50);
			graphics.endFill();
			
			_txtFps = new TextField();
			_txtMs = new TextField();
			_txtMem = new TextField();
			
			_txtFps.defaultTextFormat = _txtMs.defaultTextFormat = _txtMem.defaultTextFormat = _tfStyle;
			_txtFps.width = _txtMs.width = _txtMem.width = 60;
			_txtFps.selectable = _txtMs.selectable = _txtMem.selectable = false;
			
			_txtFps.textColor = 0xFFFF00;
			_txtFps.text = "FPS: ";
			addChild(_txtFps);
			
			_txtMs.y = 10;
			_txtMs.textColor = 0x00FF00;
			_txtMs.text = "MS: ";
			addChild(_txtMs);

			_txtMem.y = 20;
			_txtMem.textColor = 0x00FFFF;
			_txtMem.text = "MEM: ";
			addChild(_txtMem);
			
			addEventListener(MouseEvent.CLICK, mouseHandler);
			addEventListener(Event.ENTER_FRAME, update);
		} // END CONSTRUCTOR
		
		
	// **********************************************************************************
		
	
		private function mouseHandler( e:MouseEvent ):void
		{
			if (this.mouseY > this.height * .35)
				stage.frameRate --;
			else
				stage.frameRate ++;
				
			_txtFps.text = "FPS: " + _nFps + " / " + stage.frameRate;
		} // END FUNCTION mouseHandler
		
		
	// **********************************************************************************
		
	
		private function update( e:Event ):void
		{
			_nTimer = getTimer();
			_nFps++;
			
			if (_nTimer - 1000 > _nMsPrev)
			{
				_nMsPrev = _nTimer;
				_nMem = Number( ( System.totalMemory / 1048576 ).toFixed(3) );
				
				var fpsGraph : int = Math.min( 50, 50 / stage.frameRate * _nFps );
				var memGraph:Number =  Math.min( 50, Math.sqrt( Math.sqrt( _nMem * 5000 ) ) ) - 2;
				
				_bdGraph.scroll( 1, 0 );
				_bdGraph.fillRect( new Rectangle( 0, 0, 1, _bdGraph.height ), 0x333 );
				_bdGraph.setPixel( 0, _bdGraph.height - fpsGraph, 0xFFFF00);
				_bdGraph.setPixel( 0, _bdGraph.height - ( ( _nTimer - _nMs ) >> 1 ), 0x00FF00 );				
				_bdGraph.setPixel( 0, _bdGraph.height - memGraph, 0x00FFFF);
				
				_txtFps.text = "FPS: " + _nFps + " / " + stage.frameRate;
				_txtMem.text = "MEM: " + _nMem;
				
				_nFps = 0;
			} // END IF
			
			_txtMs.text = "MS: " + (_nTimer - _nMs);
			_nMs = _nTimer;
		} // END FUNCTION update
		
		
// **********************************************************************************
// **********************************************************************************


	} // END CLASS DiagnosticStats
} // END PACKAGE

